@@ -1,6 +1,6 @@
-# This file is part of the Frescobaldi project, http://www.frescobaldi.org/
+# This file is part of python-ly, https://pypi.python.org/pypi/python-ly
 #
-# Copyright (c) 2008 - 2014 by Wilbert Berendsen
+# Copyright (c) 2008 - 2015 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -46,12 +46,12 @@ except NameError:
 import fractions
 import re
 
-import ly.node
 import ly.pitch
 import ly.duration
+from ly.node import WeakNode
 
 
-class LyNode(ly.node.WeakNode):
+class LyNode(WeakNode):
     """
     Base class for LilyPond objects, based on Node,
     which takes care of the tree structure.
@@ -193,8 +193,8 @@ class Reference(object):
 
 
 class Named(object):
-    """
-    Mixin to print a \\name before the contents of the container.
+    r"""
+    Mixin to print a \name before the contents of the container.
     format() is called on the self.name attribute, so it may also
     be a Reference.
     """
@@ -206,13 +206,13 @@ class Named(object):
         
 class HandleVars(object):
     """
-    A powerful mixin class that makes handling unique variable assignments
-    inside a Container more easy.
+    A powerful mixin class to facilitate handling unique variable assignments
+    inside a Container more.
     E.g.:
     >>> h = Header()
     >>> h['composer'] = "Johann Sebastian Bach"
     creates a subnode (by default Assignment) with the name 'composer', and
-    that node again gets an autogenerated subnode of type QuotedString (If the
+    that node again gets an autogenerated subnode of type QuotedString (if the
     argument wasn't already a Node).
     """
     childClass = None # To be filled in later
@@ -375,7 +375,7 @@ class Version(Line):
 
 
 class Include(Line):
-    """ a LilyPond \\include statement """
+    r""" a LilyPond \include statement """
     def ly(self, printer):
         return r'\include "{0}"'.format(self.text)
 
@@ -496,7 +496,7 @@ class SchemeList(Enclosed):
     pre, post = "(", ")"
     
     def ly(self, printer):
-        return self.pre + super(Enclosed, self).ly(printer) + self.post
+        return self.pre + Container.ly(self, printer) + self.post
 
 
 class StatementEnclosed(Named, Enclosed):
@@ -557,8 +557,8 @@ class ContextName(Text):
 
 
 class Context(HandleVars, Section):
-    """
-    A \\context section for use inside Layout or Midi sections.
+    r"""
+    A \context section for use inside Layout or Midi sections.
     """
     name = 'context'
     
@@ -569,12 +569,12 @@ class Context(HandleVars, Section):
             
 
 class ContextType(Container):
-    """
-    \\new or \\context Staff = 'bla' \\with { } << music >>
+    r"""
+    \new or \context Staff = 'bla' \with { } << music >>
 
-    A \\with (With) element is added automatically as the first child as soon
+    A \with (With) element is added automatically as the first child as soon
     as you use our convenience methods that manipulate the variables
-    in \\with. If the \\with element is empty, it does not print anything.
+    in \with. If the \with element is empty, it does not print anything.
     You should add one other music object to this.
     """
     before, after = 1, 1
@@ -638,12 +638,12 @@ class NoteNames(ContextType): pass
 class PianoStaff(ContextType): pass
 class RhythmicStaff(ContextType): pass
 class ScoreContext(ContextType):
-    """
+    r"""
     Represents the Score context in LilyPond, but the name would
-    collide with the Score class that represents \\score { } constructs.
+    collide with the Score class that represents \score { } constructs.
 
     Because the latter is used more often, use ScoreContext to get
-    \\new Score etc.
+    \new Score etc.
     """
     ctype = 'Score'
 
@@ -657,9 +657,9 @@ class Voice(ContextType): pass
 
 
 class UserContext(ContextType):
-    """
+    r"""
     Represents a context the user creates.
-    e.g. \\new MyStaff = cid << music >>
+    e.g. \new MyStaff = cid << music >>
     """
     def __init__(self, ctype, cid=None, new=True, parent=None):
         super(UserContext, self).__init__(cid, new, parent)
@@ -724,7 +724,7 @@ class LyricsTo(LyricMode):
     def ly(self, printer):
         res = ["\\" + self.name]
         res.append(printer.quoteString(format(self.cid)))
-        res.append(super(Named, self).ly(printer))
+        res.append(Enclosed.ly(self, printer))
         return " ".join(res)
         
 
@@ -756,9 +756,9 @@ class Pitch(Leaf):
 
 
 class Duration(Leaf):
-    """
+    r"""
     A duration with duration (in logarithmical form): (-2 ... 8),
-    where -2 = \\longa, -1 = \\breve, 0 = 1, 1 = 2, 2 = 4, 3 = 8, 4 = 16, etc,
+    where -2 = \longa, -1 = \breve, 0 = 1, 1 = 2, 2 = 4, 3 = 8, 4 = 16, etc,
     dots (number of dots),
     factor (Fraction giving the scaling of the duration).
     """
@@ -790,8 +790,8 @@ class Chord(Container):
 
 
 class Relative(Statement):
-    """
-    \\relative <pitch> music
+    r"""
+    \relative <pitch> music
 
     You should add a Pitch (optionally) and another music object,
     e.g. Sim or Seq, etc.
@@ -800,18 +800,18 @@ class Relative(Statement):
 
 
 class Transposition(Statement):
-    """
-    \\transposition <pitch>
+    r"""
+    \transposition <pitch>
     You should add a Pitch.
     """
     name = 'transposition'
 
 
 class KeySignature(Leaf):
-    """
+    r"""
     A key signature expression, like:
 
-    \\key c \\major
+    \key c \major
     The pitch should be given in the arguments note and alter and is written
     out in the document's language.
     """
@@ -827,8 +827,8 @@ class KeySignature(Leaf):
 
 
 class TimeSignature(Leaf):
-    """
-    A time signature, like: \\time 4/4
+    r"""
+    A time signature, like: \time 4/4
     """
     def __init__(self, num, beat, parent=None):
         super(TimeSignature, self).__init__(parent)
@@ -840,8 +840,8 @@ class TimeSignature(Leaf):
 
 
 class Partial(Named, Duration):
-    """
-    \\partial <duration>
+    r"""
+    \partial <duration>
     You should add a Duration element
     """
     name = "partial"
@@ -849,8 +849,8 @@ class Partial(Named, Duration):
     
     
 class Tempo(Container):
-    """
-    A tempo setting, like: \\tempo 4 = 100
+    r"""
+    A tempo setting, like: \tempo 4 = 100
     May have a child markup or quoted string.
     """
     before, after = 1, 1
@@ -883,23 +883,23 @@ class Clef(Leaf):
 
 
 class VoiceSeparator(Leaf):
-    """
-    A Voice Separator: \\\\
+    r"""
+    A Voice Separator: \\
     """
     def ly(self, printer):
         return r'\\'
 
 
 class Mark(Statement):
-    """
-    The \\mark command.
+    r"""
+    The \mark command.
     """
     name = 'mark'
 
 
 class Markup(StatementEnclosed):
-    """
-    The \\markup command.
+    r"""
+    The \markup command.
     You can add many children, in that case Markup automatically prints
     { and } around them.
     """
