@@ -1,6 +1,6 @@
-# This file is part of the Frescobaldi project, http://www.frescobaldi.org/
+# This file is part of python-ly, https://pypi.python.org/pypi/python-ly
 #
-# Copyright (c) 2014 - 2014 by Wilbert Berendsen
+# Copyright (c) 2014 - 2015 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -39,6 +39,7 @@ its child nodes) ends.
 """
 
 from __future__ import unicode_literals
+from __future__ import division
 
 import itertools
 from fractions import Fraction
@@ -110,7 +111,7 @@ class dispatcher_class(dispatcher):
 
 
 class Reader(object):
-    """Reads tokens from a Source and builds a meaningful tree stucture."""
+    """Reads tokens from a Source and builds a meaningful tree structure."""
     
     _commands = dispatcher()
     _keywords = dispatcher()
@@ -130,9 +131,9 @@ class Reader(object):
         self.prev_duration = Fraction(1, 4), 1
     
     def set_language(self, lang):
-        """Changes the pitch name language to use.
+        r"""Changes the pitch name language to use.
         
-        Called internally when \language or \include tokens are encoutered
+        Called internally when \language or \include tokens are encountered
         with a valid language name/file.
         
         Sets the language attribute to the language name.
@@ -434,17 +435,17 @@ class Reader(object):
                 return
     
     def test_music_list(self, t):
-        """Test whether a music list ({ ... }, << ... >>, starts here.
+        r"""Test whether a music list ({ ... }, << ... >>, starts here.
         
-        Also handles \\simultaneous { ... } and \\sequential { ... } 
+        Also handles \simultaneous { ... } and \sequential { ... }
         correctly. These obscure commands are not even highlighted by 
-        lex, but they exist in LilyPond... \\\simultaneous { ... } is 
-        like << ... >> but \\sequential << ... >> just behaves like << ... >>
+        lex, but they exist in LilyPond... \simultaneous { ... } is
+        like << ... >> but \sequential << ... >> just behaves like << ... >>
 
         Returns a two-tuple(item; iterable), both may be None. If 
         item is not None, it can be either a UserCommand or a MusicList.  If 
-        iterable is None, the item is a UserCommand (namely \\simultaneous 
-        or \\sequential, but not followed by a { or <<); else the item is a 
+        iterable is None, the item is a UserCommand (namely \simultaneous
+        or \sequential, but not followed by a { or <<); else the item is a
         MusicList, and the iterable should be read fully to get all the 
         tokens inside the MusicList. If item is None, there is no MusicList 
         and no token is read.
@@ -473,7 +474,7 @@ class Reader(object):
         return None, None
                     
     def read_music_item(self, t, source):
-        """Read one music item (note, rest, s, \skip, or q) from t and source."""
+        r"""Read one music item (note, rest, s, \skip, or q) from t and source."""
         item = None
         in_pitch_command = isinstance(self.source.state.parser(), lilypond.ParsePitchCommand)
         if t.__class__ == lilypond.Note:
