@@ -1,6 +1,6 @@
-# This file is part of the Frescobaldi project, http://www.frescobaldi.org/
+# This file is part of python-ly, https://pypi.python.org/pypi/python-ly
 #
-# Copyright (c) 2008 - 2014 by Wilbert Berendsen
+# Copyright (c) 2008 - 2015 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,11 +18,21 @@
 # See http://www.gnu.org/licenses/ for more information.
 
 """
-Export to Music XML
-Uses xml.etree to create the XML document
+Uses xml.etree to create a Music XML document.
+
+Example::
+
+    musxml = create_musicxml.CreateMusicXML()
+    musxml.create_part()
+    musxml.create_measure(divs=1)
+    musxml.new_note('C', 4, 'whole', 4)
+    xml = musxml.musicxml()
+    xml.write(filename)
+
 """
 
 from __future__ import unicode_literals
+from __future__ import division
 
 import sys
 try:
@@ -35,7 +45,7 @@ import ly.pkginfo
 
 
 class CreateMusicXML():
-    """ creates the XML-file from the source code according to the Music XML standard """
+    """ Creates the XML nodes according to the Music XML standard."""
 
     def __init__(self):
         """Creates the basic structure of the XML without any music."""
@@ -79,8 +89,8 @@ class CreateMusicXML():
             group_symbol = etree.SubElement(partgroup, "group-symbol")
             group_symbol.text = symbol
 
-    def create_part(self, name, abbr, midi):
-        """ create a new part """
+    def create_part(self, name=False, abbr=False, midi=False):
+        """Create a new part """
         strnr = str(self.part_count)
         part = etree.SubElement(self.partlist, "score-part", id="P"+strnr)
         if name:
@@ -102,40 +112,43 @@ class CreateMusicXML():
         self.part_count += 1
         self.bar_nr = 1
 
-    def create_measure(self):
-        """ create new measure """
+    def create_measure(self, **bar_attrs):
+        """Create new measure """
         self.current_bar = etree.SubElement(self.current_part, "measure", number=str(self.bar_nr))
         self.bar_nr +=1
+        if bar_attrs:
+            self.new_bar_attr(**bar_attrs)
 
     ##
     # High-level node creation
     ##
 
-    def new_note(self, grace, pitch, base_scaling, voice, durtype, divs, dot, chord):
-        """ create all nodes needed for a note. """
+    def new_note(self, step, octave, durtype, divdur, alter=0,
+                 acc_token=0, voice=1, dot=0, chord=0, grace=(0, 0)):
+        """Create all nodes needed for a note. """
         self.create_note()
         if grace[0]:
             self.add_grace(grace[1])
         if chord:
             self.add_chord()
-        self.add_pitch(pitch[0], pitch[1], pitch[2])
+        self.add_pitch(step, alter, octave)
         if not grace[0]:
-            self.add_div_duration(self.count_duration(base_scaling, divs))
-        self.add_voice(voice)
+            self.add_div_duration(divdur)
         self.add_duration_type(durtype)
+        self.add_voice(voice)
         if dot:
             for i in range(dot):
                 self.add_dot()
-        if pitch[1] or pitch[3]:
-            if pitch[3] == '!': # cautionary
-                self.add_accidental(pitch[1], caut=True)
-            elif pitch[3] == '?': # parentheses
-                self.add_accidental(pitch[1], parenth=True)
+        if alter or acc_token:
+            if acc_token == '!': # cautionary
+                self.add_accidental(alter, caut=True)
+            elif acc_token == '?': # parentheses
+                self.add_accidental(alter, parenth=True)
             else:
-                self.add_accidental(pitch[1])
+                self.add_accidental(alter)
 
     def tuplet_note(self, fraction, base_scaling, ttype, divs):
-        """ convert current note to tuplet """
+        """Convert current note to tuplet """
         base = base_scaling[0]
         scaling = base_scaling[1]
         a = divs*4*fraction[1]
@@ -152,14 +165,14 @@ class CreateMusicXML():
         self.add_notations()
         self.add_tied(tie_type)
 
-    def new_rest(self, base_scaling, durtype, divs, pos, dot, voice):
-        """ create all nodes needed for a rest. """
+    def new_rest(self, duration, durtype, pos, dot, voice):
+        """Create all nodes needed for a rest. """
         self.create_note()
         if pos:
             self.add_rest_w_pos(pos[0], pos[1])
         else:
             self.add_rest()
-        self.add_div_duration(self.count_duration(base_scaling, divs))
+        self.add_div_duration(duration)
         self.add_voice(voice)
         if durtype:
             self.add_duration_type(durtype)
@@ -167,34 +180,28 @@ class CreateMusicXML():
             for i in range(dot):
                 self.add_dot()
 
-    def new_skip(self, base_scaling, divs):
-        base = base_scaling[0]
-        scaling = base_scaling[1]
-        duration = divs*4*base*scaling
-        self.add_skip(duration)
-
     def new_articulation(self, artic):
-        """ Add specified articulation. """
+        """Add specified articulation. """
         self.add_notations()
         self.add_articulations()
         self.add_named_artic(artic)
 
     def new_simple_ornament(self, ornament):
-        """ Add specified ornament. """
+        """Add specified ornament. """
         self.add_notations()
         self.add_ornaments()
         func_call = getattr(self, 'add_'+ornament)
         func_call()
 
     def new_adv_ornament(self, ornament, args):
-        """ Add more complex ornament."""
+        """Add more complex ornament."""
         self.add_notations()
         self.add_ornaments()
         if ornament == "wavy-line":
             self.add_wavyline(args['type'])
 
-    def new_bar_attr(self, clef, mustime, key, mode, divs):
-        """ create all bar attributes set. """
+    def new_bar_attr(self, clef=0, mustime=0, key=None, mode=0, divs=0):
+        """Create all bar attributes set. """
         self.create_bar_attr()
         if divs:
             self.add_divisions(divs)
@@ -206,34 +213,24 @@ class CreateMusicXML():
             sign, line, octch = clef
             self.add_clef(sign, line, oct_ch=octch)
 
-    def new_backup(self, base_scaling, divs):
-        self.add_backup(self.count_duration(base_scaling, divs))
-
-    def create_tempo(self, metronome, sound, dots):
+    def create_tempo(self, words, metronome, sound, dots):
         self.add_direction()
-        self.add_metron_dir(metronome[0], metronome[1], dots)
-        self.add_sound_dir(sound)
+        if words:
+            self.add_dirwords(words)
+        if metronome:
+            self.add_metron_dir(metronome[0], metronome[1], dots)
+            self.add_sound_dir(sound)
 
     def create_new_node(self, parentnode, nodename, txt):
         """ The Music XML language is extensive.
         This function can be used to create
         a non basic node not covered elsewhere in this script.
+
         TODO: add attributes
         """
         new_node = etree.SubElement(parentnode, nodename)
         new_node.text = str(txt)
 
-    ##
-    # Help functions
-    ##
-
-    def count_duration(self, base_scaling, divs):
-        base = base_scaling[0]
-        scaling = base_scaling[1]
-        duration = divs*4*base
-        duration = duration * scaling
-        return int(duration)
-
 
     ##
     # Low-level node creation
@@ -249,7 +246,7 @@ class CreateMusicXML():
         self.create_score_info("rights", rights)
 
     def create_note(self):
-        """ create new note """
+        """Create new note."""
         self.current_note = etree.SubElement(self.current_bar, "note")
         self.current_notation = None
         self.current_artic = None
@@ -257,7 +254,7 @@ class CreateMusicXML():
         self.current_tech = None
 
     def add_pitch(self, step, alter, octave):
-        """ create new pitch """
+        """Create new pitch."""
         pitch = etree.SubElement(self.current_note, "pitch")
         stepnode = etree.SubElement(pitch, "step")
         stepnode.text = str(step)
@@ -268,7 +265,7 @@ class CreateMusicXML():
         octnode.text = str(octave)
 
     def add_accidental(self, alter, caut=False, parenth=False):
-        """ create accidental """
+        """Create accidental."""
         attrib = {}
         if caut:
             attrib['cautionary'] = 'yes'
@@ -305,34 +302,34 @@ class CreateMusicXML():
         dura_node.text = str(duration)
 
     def add_div_duration(self, divdur):
-        """ create new duration """
+        """Create new duration """
         self.duration = etree.SubElement(self.current_note, "duration")
         self.duration.text = str(divdur)
 
     def change_div_duration(self, newdura):
-        """ set new duration when tuplet """
+        """Set new duration when tuplet """
         self.duration.text = str(newdura)
 
     def add_duration_type(self, durtype):
-        """ create new type """
+        """Create new type """
         typenode = etree.SubElement(self.current_note, "type")
         typenode.text = str(durtype)
 
     def add_dot(self):
-        """ create a dot """
+        """Create a dot """
         etree.SubElement(self.current_note, "dot")
 
     def add_beam(self, nr, beam_type):
-        """ Add beam. """
+        """Add beam. """
         beam_node = etree.SubElement(self.current_notation, "beam", number=str(nr))
         beam_node.text = beam_type
 
     def add_tie(self, tie_type):
-        """ create node tie (used for sound of tie) """
+        """Create node tie (used for sound of tie) """
         etree.SubElement(self.current_note, "tie", type=tie_type)
 
     def add_grace(self, slash):
-        """ create grace node """
+        """Create grace node """
         if slash:
             etree.SubElement(self.current_note, "grace", slash="yes")
         else:
@@ -343,11 +340,11 @@ class CreateMusicXML():
             self.current_notation = etree.SubElement(self.current_note, "notations")
 
     def add_tied(self, tie_type):
-        """ create node tied (used for notation of tie) """
+        """Create node tied (used for notation of tie) """
         etree.SubElement(self.current_notation, "tied", type=tie_type)
 
     def add_time_modify(self, fraction):
-        """ create time modification """
+        """Create time modification """
         timemod_node = etree.SubElement(self.current_note, "time-modification")
         actual_notes = etree.SubElement(timemod_node, "actual-notes")
         actual_notes.text = str(fraction[0])
@@ -355,26 +352,26 @@ class CreateMusicXML():
         norm_notes.text = str(fraction[1])
 
     def add_tuplet_type(self, ttype):
-        """ create tuplet with type attribute """
+        """Create tuplet with type attribute """
         etree.SubElement(self.current_notation, "tuplet", type=ttype)
 
     def add_slur(self, nr, sl_type):
-        """ Add slur. """
+        """Add slur. """
         self.add_notations()
         etree.SubElement(self.current_notation, "slur", {'number': str(nr), 'type': sl_type })
 
     def add_named_notation(self, notate):
-        """ Fermata, etc. """
+        """Fermata, etc. """
         self.add_notations()
         etree.SubElement(self.current_notation, notate)
 
     def add_articulations(self):
-        """ Common for all articulations. """
+        """Common function for creating all types of articulations. """
         if not self.current_artic:
             self.current_artic = etree.SubElement(self.current_notation, "articulations")
 
     def add_named_artic(self, artic):
-        """ Add articulation with specified name. """
+        """Add articulation with specified name. """
         etree.SubElement(self.current_artic, artic)
 
     def add_ornaments(self):
@@ -419,7 +416,7 @@ class CreateMusicXML():
         fing_node.text = str(finger_nr)
 
     def create_bar_attr(self):
-        """ create node attributes """
+        """Create node attributes """
         self.bar_attr = etree.SubElement(self.current_bar, "attributes")
 
     def add_divisions(self, div):
@@ -507,6 +504,12 @@ class CreateMusicXML():
         dirtypenode = etree.SubElement(direction, "direction-type")
         dyn_node = etree.SubElement(dirtypenode, "octave-shift", oct_dict)
 
+    def add_dirwords(self, words):
+        """Add words in direction, e. g. a tempo mark."""
+        dirtypenode = etree.SubElement(self.direction, "direction-type")
+        wordsnode = etree.SubElement(dirtypenode, "words")
+        wordsnode.text = words
+
     def add_metron_dir(self, unit, beats, dots):
         dirtypenode = etree.SubElement(self.direction, "direction-type")
         metrnode = etree.SubElement(dirtypenode, "metronome")
@@ -550,16 +553,16 @@ class MusicXML(object):
         self.root = tree.getroot()
 
     def indent(self, indent="  "):
-        """ add indent and linebreaks to the created XML tree """
+        """Add indent and linebreaks to the created XML tree."""
         import ly.etreeutil
         ly.etreeutil.indent(self.root, indent)
 
     def tostring(self, encoding='UTF-8'):
-        """ output etree as a XML document """
+        """Output etree as a XML document."""
         return etree.tostring(self.root, encoding=encoding, method="xml")
 
     def write(self, file, encoding='UTF-8', doctype=True):
-        """ write XML to a file (file obj or filename) """
+        """Write XML to a file (file obj or filename)."""
         def write(f):
             if doctype:
                 f.write((xml_decl_txt + "\n").format(encoding=encoding).encode(encoding))
@@ -567,14 +570,13 @@ class MusicXML(object):
                 self.tree.write(f, encoding=encoding, xml_declaration=False)
             else:
                 self.tree.write(f, encoding=encoding, xml_declaration=True, method="xml")
-        try:
-            file.write
-        except AttributeError:
+        if hasattr(file, 'write'):
+            write(file)
+            # do not close if it already was a file object
+        else:
             # it is not a file object
             with open(file, 'wb') as f:
                 write(f)
-        else:
-            write(file) # do not close if it already was a file object
 
 
 xml_decl_txt = """<?xml version="1.0" encoding="{encoding}"?>"""
