@@ -1,6 +1,6 @@
-# This file is part of the Frescobaldi project, http://www.frescobaldi.org/
+# This file is part of python-ly, https://pypi.python.org/pypi/python-ly
 #
-# Copyright (c) 2008 - 2014 by Wilbert Berendsen
+# Copyright (c) 2008 - 2015 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,11 +19,12 @@
 
 """
 The information of the parsed source is organised into an object
-structure suitable for the transformation into a XML tree.
+structure with help of the classes in ly.musicxml.xml_objs.
 """
 
 from __future__ import unicode_literals
 from __future__ import print_function
+from __future__ import division
 
 from fractions import Fraction
 
@@ -34,7 +35,8 @@ from . import xml_objs
 
 
 class Mediator():
-    """ Help class between the ly source parser and the XML creator """
+    """Help class that acts as mediator between the ly source parser
+    and the XML creating modules."""
 
     def __init__(self):
         """ create global lists """
@@ -56,6 +58,7 @@ class Mediator():
         self.group = None
         self.group_num = 0
         self.current_chord = []
+        self.q_chord = []
         self.prev_pitch = None
         self.prev_chord_pitch = None
         self.store_voicenr = 0
@@ -67,6 +70,10 @@ class Mediator():
         self.lyric_syll = False
         self.lyric_nr = 1
         self.ongoing_wedge = False
+        self.octdiff = 0
+        self.prev_tremolo = 8
+        self.tupl_dur = 0
+        self.tupl_sum = 0
 
     def new_header_assignment(self, name, value):
         """Distributing header information."""
@@ -438,8 +445,11 @@ class Mediator():
         return chord_note
 
     def copy_prev_chord(self, duration):
-        prev_chord = self.current_chord
-        self.clear_chord()
+        if self.current_chord:
+            prev_chord = self.current_chord
+            self.clear_chord()
+        else:
+            prev_chord = self.q_chord
         for i, pc in enumerate(prev_chord):
             cn = self.copy_barnote_basics(pc)
             cn.set_duration(duration)
@@ -453,6 +463,7 @@ class Mediator():
         self.tied = False
 
     def clear_chord(self):
+        self.q_chord = self.current_chord
         self.current_chord = []
         self.prev_chord_pitch = None
 
@@ -469,7 +480,7 @@ class Mediator():
             self.current_note = xml_objs.BarRest(dur, self.voice)
         elif rtype == 'R':
             self.current_note = xml_objs.BarRest(dur, self.voice, show_type=False)
-        elif rtype == 's' or rtype == '\skip':
+        elif rtype == 's' or rtype == '\\skip':
             self.current_note = xml_objs.BarRest(dur, self.voice, skip=True)
         self.check_current_note(rest=True)
 
@@ -494,9 +505,36 @@ class Mediator():
             rest_copy = xml_objs.BarRest(dur, voice=voc, show_type=st, skip=sk)
             self.add_to_bar(rest_copy)
 
-    def change_to_tuplet(self, tfraction, ttype):
+    def change_to_tuplet(self, tfraction, ttype, check_dur=True):
+        """Change the current note into a tuplet note."""
+        tuplscaling = Fraction(tfraction[0], tfraction[1])
+        if check_dur and self.tupl_dur:
+            if self.tupl_sum == 0:
+                ttype = "start"
+            base, scaling = self.current_lynote.duration
+            self.tupl_sum += (1 / tuplscaling) * base * scaling
+            if self.tupl_sum == self.tupl_dur:
+                ttype = "stop"
+                self.tupl_sum = 0
         self.current_note.set_tuplet(tfraction, ttype)
-        self.check_divs(Fraction(tfraction[0], tfraction[1]))
+        self.check_divs(tuplscaling)
+
+    def set_tuplspan_dur(self, token=None, tokens=None, fraction=None):
+        """
+        Catch duration set by the tupletSpannerDuration property.
+
+        Set the fraction directly or calculate it from tokens.
+        """
+        if fraction:
+            self.tupl_dur = fraction
+        else:
+            base, scaling = ly.duration.base_scaling((token,) + tokens)
+            self.tupl_dur = base * scaling
+
+    def unset_tuplspan_dur(self):
+        """Reset tuplet duration sum and tuplet spanner duration."""
+        self.tupl_sum = 0
+        self.tupl_dur = 0
 
     def tie_to_next(self):
         tie_type = 'start'
@@ -528,17 +566,17 @@ class Mediator():
     def new_dynamics(self, dynamics):
         hairpins = {'<': 'crescendo', '>': 'diminuendo'}
         if dynamics == '!':
-            self.current_note.set_dynamics_before(wedge='stop')
+            self.current_note.set_dynamics(wedge='stop')
             self.ongoing_wedge = False
         elif dynamics in hairpins:
-            self.current_note.set_dynamics_after(wedge=hairpins[dynamics])
+            self.current_note.set_dynamics(wedge=hairpins[dynamics])
             self.ongoing_wedge = True
         elif self.ongoing_wedge:
-            self.current_note.set_dynamics_before(wedge='stop')
-            self.current_note.set_dynamics_after(mark=dynamics)
+            self.current_note.set_dynamics(wedge='stop')
+            self.current_note.set_dynamics(mark=dynamics)
             self.ongoing_wedge = False
         else:
-            self.current_note.set_dynamics_before(mark=dynamics)
+            self.current_note.set_dynamics(mark=dynamics)
 
     def new_grace(self, slash=0):
         self.current_note.set_grace(slash)
@@ -567,11 +605,15 @@ class Mediator():
         if self.current_note.tremolo[1]: #tremolo already set
             self.current_note.set_tremolo(trem_type)
         else:
-            if not duration:
+            if repeats:
                 duration = int(self.dur_token)
                 bs, durtype = calc_trem_dur(repeats, self.current_note.duration, duration)
                 self.current_note.duration = bs
                 self.current_note.type = durtype
+            elif not duration:
+                duration = self.prev_tremolo
+            else:
+                self.prev_tremolo = duration
             self.current_note.set_tremolo(trem_type, duration)
 
     def new_trill_spanner(self, end=None):
@@ -581,26 +623,33 @@ class Mediator():
         self.current_note.add_adv_ornament('wavy-line', end)
 
     def new_ottava(self, octdiff):
-        nr = int(octdiff)
-        if nr == 0:
-            size = self.oct_size
-            plac = self.oct_plac
-            octdir = "stop"
+        octdiff = int(octdiff)
+        if self.octdiff == octdiff:
+            return
+        if octdiff == 0:
+            if self.octdiff < 0:
+                plac = "below"
+            else:
+                plac = "above"
+            size = abs(self.octdiff) * 7 + 1
+            self.current_note.set_oct_shift(plac, "stop", size)
         else:
-            if nr < 0:
+            if octdiff < 0:
                 plac = "below"
                 octdir = "up"
-            elif nr > 0:
+            else:
                 plac = "above"
                 octdir = "down"
-            size = abs(nr) * 7 + 1
-            self.oct_size = size
-            self.oct_plac = plac
-        self.current_note.set_oct_shift(plac, octdir, size)
+            size = abs(octdiff) * 7 + 1
+            self.current_note.set_oct_shift(plac, octdir, size)
+        self.octdiff = octdiff
 
-    def new_tempo(self, dur_tokens, tempo, string):
-        unit, dots, rs = self.duration_from_tokens(dur_tokens)
-        beats = tempo[0]
+    def new_tempo(self, unit, dur_tokens, tempo, string):
+        dots, rs = self.duration_from_tokens(dur_tokens)
+        if tempo:
+            beats = tempo[0]
+        else:
+            beats = 0
         try:
             text = string.value()
         except AttributeError:
@@ -612,7 +661,7 @@ class Mediator():
         self.bar.add(tempo)
 
     def set_by_property(self, prprty, value, group=False):
-        """Generic setter for deferent properties."""
+        """Generic setter for different properties."""
         if prprty == 'instrumentName':
             if group:
                 self.set_groupname(value)
@@ -712,9 +761,9 @@ class Mediator():
             a = 4*den
             b = (1/base)*num
         c = a*divs
-        predur, mod = divmod(c,b)
+        predur, mod = divmod(c, b)
         if mod > 0:
-            mult = get_mult(a,b)
+            mult = get_mult(a, b)
             self.divisions = divs*mult
 
 
@@ -759,17 +808,17 @@ def clefname2clef(clefname):
     Add it to the python dictionary below.
     """
     clef_dict = {
-    "treble": ('G',2,0), "violin": ('G',2,0), "G": ('G',2,0),
-    "bass": ('F',4,0), "F": ('F',4,0),
-    "alto": ('C',3,0), "C": ('C',3,0),
-    "tenor": ('C',4,0), "treble_8": ('G',2,-1),
-    "bass_8": ('F',4,-1), "treble^8": ('G',2,1),
-    "bass^8": ('F',4,1), "percussion": ('percussion',0,0),
-    "tab": ('TAB',5,0), "soprano": ('C',1,0),
-    "mezzosoprano": ('C',2,0),
-    "baritone": ('C',5,0),
-    "varbaritone": ('F',3,0),
-    "french": ('G',1,0)
+    "treble": ('G', 2, 0), "violin": ('G', 2, 0), "G": ('G', 2, 0),
+    "bass": ('F', 4, 0), "F": ('F', 4, 0),
+    "alto": ('C', 3, 0), "C": ('C', 3, 0),
+    "tenor": ('C', 4, 0), "treble_8": ('G', 2, -1),
+    "bass_8": ('F', 4, -1), "treble^8": ('G', 2, 1),
+    "bass^8": ('F', 4, 1), "percussion": ('percussion', 0, 0),
+    "tab": ('TAB', 5, 0), "soprano": ('C', 1, 0),
+    "mezzosoprano": ('C', 2, 0),
+    "baritone": ('C', 5, 0),
+    "varbaritone": ('F', 3, 0),
+    "french": ('G', 1, 0)
     }
     try:
         clef = clef_dict[clefname]
@@ -818,7 +867,12 @@ def calc_trem_dur(repeats, base_scaling, duration):
     base = base_scaling[0]
     scale = base_scaling[1]
     new_base = base * repeats
-    new_type = xml_objs.durval2type(str(duration/repeats))
+    if repeats > duration:
+        import ly.duration
+        trem_length = ly.duration.tostring(int((repeats // duration) * -0.5))
+    else:
+        trem_length = str(duration // repeats)
+    new_type = xml_objs.durval2type(trem_length)
     return (new_base, scale), new_type
 
 def get_line_style(style):
