@@ -1,6 +1,6 @@
-# This file is part of the Frescobaldi project, http://www.frescobaldi.org/
+# This file is part of python-ly, https://pypi.python.org/pypi/python-ly
 #
-# Copyright (c) 2008 - 2014 by Wilbert Berendsen
+# Copyright (c) 2008 - 2015 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,17 +18,12 @@
 # See http://www.gnu.org/licenses/ for more information.
 
 """
-Export to Music XML.
+Using the tree structure from ly.music to initiate the conversion to MusicXML.
 
-Using ly.music document tree to convert the source to XML.
-
-Uses functions similar to items.Document.iter_music() to iter through
+Uses functions similar to ly.music.items.Document.iter_music() to iter through
 the node tree. But information about where a node branch ends
-is also added.
-
-This approach substitues the previous method of using ly.lex directly
-to parse the source. It should prove more stable and easier to implement.
-
+is also added. During the iteration the information needed for the conversion
+is captured.
 """
 
 from __future__ import unicode_literals
@@ -37,8 +32,8 @@ from __future__ import print_function
 import ly.music
 
 from . import create_musicxml
-from . import iter_mediator
 from . import ly2xml_mediator
+from . import xml_objs
 
 #excluded from parsing
 excl_list = ['Version', 'Midi', 'Layout']
@@ -81,6 +76,10 @@ class ParseSource():
         self.ottava = False
         self.with_contxt = None
         self.schm_assignm = None
+        self.tempo = ()
+        self.tremolo = False
+        self.tupl_span = False
+        self.unset_tuplspan = False
 
     def parse_tree(self, mustree):
         # print(mustree.dump())
@@ -115,7 +114,7 @@ class ParseSource():
 
     def musicxml(self, prettyprint=True):
         self.mediator.check_score()
-        iter_mediator.iterateMediatorScore(
+        xml_objs.IterateXmlObjs(
             self.mediator.score, self.musxml, self.mediator.divisions)
         xml = self.musxml.musicxml(prettyprint)
         return xml
@@ -165,7 +164,7 @@ class ParseSource():
         self.mediator.copy_prev_chord(q.duration)
 
     def Context(self, context):
-        """ \context """
+        r""" \context """
         self.in_context = True
         if context.context() in pno_contexts:
             self.mediator.new_part(piano=True)
@@ -198,7 +197,7 @@ class ParseSource():
         self.mediator.set_voicenr(add=True)
 
     def Change(self, change):
-        """ A \\change music expression. Changes the staff number. """
+        r""" A \change music expression. Changes the staff number. """
         if change.context() == 'Staff':
             self.mediator.set_staffnr(0, staff_id=change.context_id())
 
@@ -207,7 +206,7 @@ class ParseSource():
         self.mediator.new_bar()
 
     def Clef(self, clef):
-        """ Clef \clef"""
+        r""" Clef \clef"""
         self.mediator.new_clef(clef.specifier())
 
     def KeySignature(self, key):
@@ -226,7 +225,7 @@ class ParseSource():
                 self.ttype = ""
             if self.grace_seq:
                 self.mediator.new_grace()
-            if self.trem_rep:
+            if self.trem_rep and not self.look_ahead(note, ly.music.items.Duration):
                 self.mediator.set_tremolo(trem_type='start', repeats=self.trem_rep)
         else:
             if isinstance(note.parent(), ly.music.items.Relative):
@@ -240,31 +239,46 @@ class ParseSource():
 
     def Duration(self, duration):
         """A written duration"""
-        self.mediator.new_duration_token(duration.token, duration.tokens)
+        if self.tempo:
+            self.mediator.new_tempo(duration.token, duration.tokens, *self.tempo)
+            self.tempo = ()
+        elif self.tremolo:
+            self.mediator.set_tremolo(duration=int(duration.token))
+            self.tremolo = False
+        elif self.tupl_span:
+            self.mediator.set_tuplspan_dur(duration.token, duration.tokens)
+            self.tupl_span = False
+        else:
+            self.mediator.new_duration_token(duration.token, duration.tokens)
+            if self.trem_rep:
+                self.mediator.set_tremolo(trem_type='start', repeats=self.trem_rep)
 
     def Tempo(self, tempo):
         """ Tempo direction, e g '4 = 80' """
-        self.mediator.new_tempo(tempo.duration.tokens, tempo.tempo(), tempo.text())
+        if self.look_ahead(tempo, ly.music.items.Duration):
+            self.tempo = (tempo.tempo(), tempo.text())
+        else:
+            self.mediator.new_tempo(0, (), tempo.tempo(), tempo.text())
 
     def Tie(self, tie):
         """ tie ~ """
         self.mediator.tie_to_next()
 
     def Rest(self, rest):
-        """ rest, r or R. Note: NOT by command, i.e. \rest """
+        r""" rest, r or R. Note: NOT by command, i.e. \rest """
         if rest.token == 'R':
             self.scale = 'R'
         self.mediator.new_rest(rest)
 
     def Skip(self, skip):
-        """ invisible rest/spacer rest (s or command \skip)"""
+        r""" invisible rest/spacer rest (s or command \skip)"""
         if 'lyrics' in self.sims_and_seqs:
             self.mediator.new_lyrics_item(skip.token)
         else:
             self.mediator.new_rest(skip)
 
     def Scaler(self, scaler):
-        """
+        r"""
         \times \tuplet \scaleDurations
 
         """
@@ -277,6 +291,9 @@ class ParseSource():
         elif scaler.token == '\\tuplet':
             self.ttype = "start"
             self.fraction = (scaler.numerator, scaler.denominator)
+        if self.look_ahead(scaler, ly.music.items.Duration):
+            self.tupl_span = True
+            self.unset_tuplspan = True
         self.tuplet = True
 
     def Number(self, number):
@@ -316,15 +333,26 @@ class ParseSource():
             self.trem_rep = repeat.repeat_count()
 
     def Tremolo(self, tremolo):
-        self.mediator.set_tremolo(duration=int(tremolo.duration.token))
+        """A tremolo item ":"."""
+        if self.look_ahead(tremolo, ly.music.items.Duration):
+            self.tremolo = True
+        else:
+            self.mediator.set_tremolo()
 
     def With(self, cont_with):
-        """A \\with ... construct."""
+        r"""A \with ... construct."""
         self.with_contxt = cont_with.parent().context()
 
     def Set(self, cont_set):
-        """A \\set command."""
+        r"""A \set command."""
         if isinstance(cont_set.value(), ly.music.items.Scheme):
+            if cont_set.property() == 'tupletSpannerDuration':
+                moment = cont_set.value().get_ly_make_moment()
+                if moment:
+                    self.mediator.set_tuplspan_dur(fraction=moment)
+                else:
+                    self.mediator.unset_tuplspan_dur()
+                return
             val = cont_set.value().get_string()
         else:
             val = cont_set.value().value()
@@ -334,7 +362,8 @@ class ParseSource():
             self.mediator.set_by_property(cont_set.property(), val, group=True)
 
     def Command(self, command):
-        """ \bar, \rest etc """
+        r""" \bar, \rest etc """
+        excls = ['\\major', '\\minor', '\\bar']
         if command.token == '\\rest':
             self.mediator.note2rest()
         elif command.token == '\\numericTimeSignature':
@@ -354,8 +383,18 @@ class ParseSource():
             self.mediator.new_trill_spanner("stop")
         elif command.token == '\\ottava':
             self.ottava = True
+        elif command.token == '\\default':
+            if self.tupl_span:
+                self.mediator.unset_tuplspan_dur()
+                self.tupl_span = False
         else:
-            print("Unknown command:", command.token)
+            if command.token not in excls:
+                print("Unknown command:", command.token)
+
+    def UserCommand(self, usercommand):
+        """Music variables are substituted so this must be something else."""
+        if usercommand.name() == 'tupletSpan':
+            self.tupl_span = True
 
     def String(self, string):
         prev = self.get_previous_node(string)
@@ -363,7 +402,7 @@ class ParseSource():
             self.mediator.create_barline(string.value())
 
     def LyricsTo(self, lyrics_to):
-        """A \\lyricsto expression. """
+        r"""A \lyricsto expression. """
         self.mediator.new_lyric_section('lyricsto'+lyrics_to.context_id(), lyrics_to.context_id())
         self.sims_and_seqs.append('lyrics')
 
@@ -376,15 +415,15 @@ class ParseSource():
         self.mediator.new_lyrics_item(lyrics_item.token)
 
     def LyricMode(self, lyric_mode):
-        """A \\lyricmode, \\lyrics or \\addlyrics expression."""
+        r"""A \lyricmode, \lyrics or \addlyrics expression."""
         pass
 
     def Override(self, override):
-        """An \\override command."""
+        r"""An \override command."""
         self.override_key = ''
 
     def PathItem(self, item):
-        """An item in the path of an \\override or \\revert command."""
+        r"""An item in the path of an \override or \revert command."""
         self.override_key += item.token
 
     def Scheme(self, scheme):
@@ -408,10 +447,13 @@ class ParseSource():
 
     def End(self, end):
         if isinstance(end.node, ly.music.items.Scaler):
-            if end.node.token == '\scaleDurations':
+            if self.unset_tuplspan:
+                self.mediator.unset_tuplspan_dur()
+                self.unset_tuplspan = False
+            if end.node.token == '\\scaleDurations':
                 self.mediator.change_to_tuplet(self.fraction, "")
             else:
-                self.mediator.change_to_tuplet(self.fraction, "stop")
+                self.mediator.change_to_tuplet(self.fraction, "stop", check_dur=False)
             self.tuplet = False
             self.fraction = None
         elif isinstance(end.node, ly.music.items.Grace): #Grace
@@ -421,9 +463,9 @@ class ParseSource():
                 self.mediator.new_repeat('backward')
             elif end.node.specifier() == 'tremolo':
                 if self.look_ahead(end.node, ly.music.items.MusicList):
-                    self.mediator.set_tremolo(trem_type="stop", repeats=self.trem_rep)
+                    self.mediator.set_tremolo(trem_type="stop")
                 else:
-                    self.mediator.set_tremolo(trem_type="single", repeats=self.trem_rep)
+                    self.mediator.set_tremolo(trem_type="single")
                 self.trem_rep = 0
         elif isinstance(end.node, ly.music.items.Context):
             self.in_context = False
@@ -499,12 +541,12 @@ class ParseSource():
         return False
 
     def iter_score(self, scorenode, doc):
-        """
+        r"""
         Iter over score.
 
         Similarly to items.Document.iter_music user commands are substituted.
 
-        Furthermore \\repeat unfold expressions are unfolded.
+        Furthermore \repeat unfold expressions are unfolded.
         """
         for s in scorenode:
             if isinstance(s, ly.music.items.Repeat) and s.specifier() == 'unfold':
@@ -519,8 +561,8 @@ class ParseSource():
             yield End(scorenode)
 
     def unfold_repeat(self, repeat_node, repeat_count, doc):
-        """
-        Iter over node which represent a \\repeat unfold expression
+        r"""
+        Iter over node which represent a \repeat unfold expression
         and do the unfolding directly.
         """
         for r in range(repeat_count):
