@@ -1,6 +1,6 @@
-# This file is part of the Frescobaldi project, http://www.frescobaldi.org/
+# This file is part of python-ly, https://pypi.python.org/pypi/python-ly
 #
-# Copyright (c) 2008 - 2014 by Wilbert Berendsen
+# Copyright (c) 2008 - 2015 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,7 +18,34 @@
 # See http://www.gnu.org/licenses/ for more information.
 
 """
-Classes that holds information suitable for converting to musicXML.
+Classes that holds information about a musical score,
+suitable for converting to musicXML.
+
+When the score structure is built, it can easily be used to create a musicXML.
+
+Example::
+
+    from ly.musicxml import create_musicxml, xml_objs
+
+    musxml = create_musicxml.CreateMusicXML()
+
+    score = xml_objs.Score()
+    part = xml_objs.ScorePart()
+    score.partlist.append(part)
+    bar = xml_objs.Bar()
+    part.barlist.append(bar)
+    ba = xml_objs.BarAttr()
+    ba.set_time([4,4])
+    bar.obj_list.append(ba)
+    c = xml_objs.BarNote('c', 0, 0, (1,1))
+    c.set_octave(4)
+    c.set_durtype(1)
+    bar.obj_list.append(c)
+
+    xml_objs.IterateXmlObjs(score, musxml, 1)
+    xml = musxml.musicxml()
+    xml.write(filename)
+
 """
 
 from __future__ import unicode_literals
@@ -26,6 +53,163 @@ from __future__ import print_function
 
 from fractions import Fraction
 
+class IterateXmlObjs():
+    """
+    A ly.musicxml.xml_objs.Score object is iterated and the Music XML node tree
+    is constructed.
+
+    """
+    def __init__(self, score, musxml, div):
+        """Create the basic score information, and initiate the
+        iteration of the parts."""
+        # score.debug_score([])
+        self.musxml = musxml
+        self.divisions = div
+        if score.title:
+            self.musxml.create_title(score.title)
+        for ctag in score.creators:
+            self.musxml.add_creator(ctag, score.creators[ctag])
+        for itag in score.info:
+            self.musxml.create_score_info(itag, score.info[itag])
+        if score.rights:
+            self.musxml.add_rights(score.rights)
+        for p in score.partlist:
+            if isinstance(p, ScorePart):
+                self.iterate_part(p)
+            elif isinstance(p, ScorePartGroup):
+                self.iterate_partgroup(p)
+
+    def iterate_partgroup(self, group):
+        """Loop through a group, recursively if nested."""
+        self.musxml.create_partgroup(
+            'start', group.num, group.name, group.abbr, group.bracket)
+        for p in group.partlist:
+            if isinstance(p, ScorePart):
+                self.iterate_part(p)
+            elif isinstance(p, ScorePartGroup):
+                self.iterate_partgroup(p)
+        self.musxml.create_partgroup('stop', group.num)
+
+    def iterate_part(self, part):
+        """The part is iterated."""
+        if part.barlist:
+            part.set_first_bar(self.divisions)
+            self.musxml.create_part(part.name, part.abbr, part.midi)
+            for bar in part.barlist:
+                self.iterate_bar(bar)
+        else:
+            print("Warning: empty part:", part.name)
+
+    def iterate_bar(self, bar):
+        """The objects in the bar are output to the xml-file."""
+        self.musxml.create_measure()
+        for obj in bar.obj_list:
+            if isinstance(obj, BarAttr):
+                self.new_xml_bar_attr(obj)
+            elif isinstance(obj, BarMus):
+                self.before_note(obj)
+                if isinstance(obj, BarNote):
+                    self.new_xml_note(obj)
+                elif isinstance(obj, BarRest):
+                    self.new_xml_rest(obj)
+                self.gener_xml_mus(obj)
+                self.after_note(obj)
+            elif isinstance(obj, BarBackup):
+                divdur = self.count_duration(obj.duration, self.divisions)
+                self.musxml.add_backup(divdur)
+
+    def new_xml_bar_attr(self, obj):
+        """Create bar attribute xml-nodes."""
+        if obj.has_attr():
+            self.musxml.new_bar_attr(obj.clef, obj.time, obj.key, obj.mode, obj.divs)
+        if obj.repeat:
+            self.musxml.add_barline(obj.barline, obj.repeat)
+        elif obj.barline:
+            self.musxml.add_barline(obj.barline)
+        if obj.staves:
+            self.musxml.add_staves(obj.staves)
+        if obj.multiclef:
+            for mc in obj.multiclef:
+                self.musxml.add_clef(sign=mc[0][0], line=mc[0][1], nr=mc[1], oct_ch=mc[0][2])
+        if obj.tempo:
+            self.musxml.create_tempo(obj.tempo.text, obj.tempo.metr,
+                                     obj.tempo.midi, obj.tempo.dots)
+
+    def before_note(self, obj):
+        """Xml-nodes before note."""
+        for d in obj.dynamic:
+            if d.before:
+                if d.is_mark:
+                    self.musxml.add_dynamic_mark(d.sign)
+                else:
+                    self.musxml.add_dynamic_wedge(d.sign)
+
+    def after_note(self, obj):
+        """Xml-nodes after note."""
+        for d in obj.dynamic:
+            if not d.before:
+                if d.is_mark:
+                    self.musxml.add_dynamic_mark(d.sign)
+                else:
+                    self.musxml.add_dynamic_wedge(d.sign)
+        if obj.oct_shift:
+            self.musxml.add_octave_shift(obj.oct_shift.plac, obj.oct_shift.octdir, obj.oct_shift.size)
+
+    def gener_xml_mus(self, obj):
+        """Nodes generic for both notes and rests."""
+        if obj.tuplet:
+            self.musxml.tuplet_note(obj.tuplet, obj.duration, obj.ttype, self.divisions)
+        if obj.staff and not obj.skip:
+            self.musxml.add_staff(obj.staff)
+        if obj.other_notation:
+            self.musxml.add_named_notation(obj.other_notation)
+
+    def new_xml_note(self, obj):
+        """Create note specific xml-nodes."""
+        divdur = self.count_duration(obj.duration, self.divisions)
+        self.musxml.new_note(obj.base_note, obj.octave, obj.type, divdur,
+            obj.alter, obj.accidental_token, obj.voice, obj.dot, obj.chord,
+            obj.grace)
+        for t in obj.tie:
+            self.musxml.tie_note(t)
+        for s in obj.slur:
+            self.musxml.add_slur(1, s) #LilyPond doesn't allow nested slurs so the number can be 1
+        for a in obj.artic:
+            self.musxml.new_articulation(a)
+        if obj.ornament:
+            self.musxml.new_simple_ornament(obj.ornament)
+        if obj.adv_ornament:
+            self.musxml.new_adv_ornament(obj.adv_ornament[0], obj.adv_ornament[1])
+        if obj.tremolo[1]:
+            self.musxml.add_tremolo(obj.tremolo[0], obj.tremolo[1])
+        if obj.gliss:
+            self.musxml.add_gliss(obj.gliss[0], obj.gliss[1], obj.gliss[2])
+        if obj.fingering:
+            self.musxml.add_fingering(obj.fingering)
+        if obj.lyric:
+            for l in obj.lyric:
+                try:
+                   self.musxml.add_lyric(l[0], l[1], l[2], l[3])
+                except IndexError:
+                    self.musxml.add_lyric(l[0], l[1], l[2])
+
+    def new_xml_rest(self, obj):
+        """Create rest specific xml-nodes."""
+        divdur = self.count_duration(obj.duration, self.divisions)
+        if obj.skip:
+            self.musxml.add_skip(divdur)
+        else:
+            self.musxml.new_rest(divdur, obj.type, obj.pos,
+            obj.dot, obj.voice)
+
+    def count_duration(self, base_scaling, divs):
+        base = base_scaling[0]
+        scaling = base_scaling[1]
+        duration = divs*4*base
+        duration = duration * scaling
+        return int(duration)
+
+
 class Score():
     """Object that keep track of a whole score."""
     def __init__(self):
@@ -99,8 +283,8 @@ class ScorePart():
         self.staves = staves
 
     def set_first_bar(self, divisions):
-        initime = [4,4]
-        iniclef = ('G',2,0)
+        initime = [4, 4]
+        iniclef = ('G', 2, 0)
 
         def check_time(bar):
             for obj in bar.obj_list:
@@ -169,7 +353,7 @@ class ScoreSection():
 
 
 class Snippet(ScoreSection):
-    """ Short section indended to be merged.
+    """ Short section intended to be merged.
     Holds reference to the barlist to be merged into."""
     def __init__(self, name, merge_into):
         ScoreSection.__init__(self, name)
@@ -212,7 +396,7 @@ class Bar():
                     s *= obj.duration[1]
             elif isinstance(obj, BarBackup):
                 break
-        self.add(BarBackup((b,s)))
+        self.add(BarBackup((b, s)))
 
     def is_skip(self):
         """ Check if bar has nothing but skips. """
@@ -258,10 +442,7 @@ class BarMus():
         self.staff = 0
         self.chord = False
         self.other_notation = None
-        self.dynamic = {
-        'before': {'mark': None, 'wedge': None },
-        'after': {'mark': None, 'wedge': None }
-        }
+        self.dynamic = []
         self.oct_shift = None
 
     def __repr__(self):
@@ -280,17 +461,14 @@ class BarMus():
     def add_other_notation(self, other):
         self.other_notation = other
 
-    def set_dynamics_before(self, mark=None, wedge=None):
+    def set_dynamics(self, mark=None, wedge=None, before=True):
         if mark:
-            self.dynamic['before']['mark'] = mark
+            sign = mark
+            is_mark = True
         if wedge:
-            self.dynamic['before']['wedge'] = wedge
-
-    def set_dynamics_after(self, mark=None, wedge=None):
-        if mark:
-            self.dynamic['after']['mark'] = mark
-        if wedge:
-            self.dynamic['after']['wedge'] = wedge
+            sign = wedge
+            is_mark = False
+        self.dynamic.append(Dynamics(sign, before, is_mark))
 
     def set_oct_shift(self, plac, octdir, size):
         self.oct_shift = OctaveShift(plac, octdir, size)
@@ -312,6 +490,14 @@ class OctaveShift():
         self.size = size
 
 
+class Dynamics():
+    """Stores information about dynamics. """
+    def __init__(self, sign, before=True, is_mark=False, ):
+        self.before = before
+        self.is_mark = is_mark
+        self.sign = sign
+
+
 ##
 # Subclasses of BarMus
 ##
@@ -321,14 +507,14 @@ class BarNote(BarMus):
     """ object to keep track of note parameters """
     def __init__(self, pitch_note, alter, accidental, duration, voice=1):
         BarMus.__init__(self, duration, voice)
-        self.base_note = pitch_note
+        self.base_note = pitch_note.upper()
         self.alter = alter
         self.octave = None
         self.accidental_token = accidental
         self.tie = []
-        self.grace = (0,0)
+        self.grace = (0, 0)
         self.gliss = None
-        self.tremolo = ('',0)
+        self.tremolo = ('', 0)
         self.skip = False
         self.slur = []
         self.artic = []
@@ -365,7 +551,7 @@ class BarNote(BarMus):
         self.adv_ornament = (ornament, {"type": end_type})
 
     def set_grace(self, slash):
-        self.grace = (1,slash)
+        self.grace = (1, slash)
 
     def set_gliss(self, line, endtype = "start", nr=1):
         if not line:
@@ -436,9 +622,9 @@ class BarAttr():
         self.key = muskey
         self.mode = mode
 
-    def set_time(self, fractlist, numeric):
+    def set_time(self, fractlist, numeric=True):
         self.time = fractlist
-        if not numeric and (fractlist == [2,2] or fractlist == [4,4]):
+        if not numeric and (fractlist == [2, 2] or fractlist == [4, 4]):
             self.time.append('common')
 
     def set_clef(self, clef):
@@ -447,7 +633,7 @@ class BarAttr():
     def set_barline(self, bl):
         self.barline = convert_barl(bl)
 
-    def set_tempo(self, unit, beats, dots=0, text=""):
+    def set_tempo(self, unit=0, beats=0, dots=0, text=""):
         self.tempo = TempoDir(unit, beats, dots, text)
 
     def has_attr(self):
@@ -474,17 +660,21 @@ class BarBackup():
 class TempoDir():
     """ Object that stores tempo direction information """
     def __init__(self, unit, beats, dots, text):
-        self.metr = durval2type(unit), beats
-        self.text = text
-        self.midi = self.set_midi_tempo(unit, beats, dots)
+        if unit:
+            self.metr = durval2type(unit), beats
+            self.midi = self.set_midi_tempo(unit, beats, dots)
+        else:
+            self.metr = 0
+            self.midi = 0
         self.dots = dots
+        self.text = text
 
     def set_midi_tempo(self, unit, beats, dots):
-        u = Fraction(1,int(unit))
+        u = Fraction(1, int(unit))
         if dots:
             import math
-            den = int(math.pow(2,dots))
-            num = int(math.pow(2,dots+1)-1)
+            den = int(math.pow(2, dots))
+            num = int(math.pow(2, dots+1)-1)
             u *= Fraction(num, den)
         mult = 4*u
         return float(Fraction(beats)*mult)
@@ -502,7 +692,11 @@ def durval2type(durval):
         "16th", "32nd", "64th",
         "128th", "256th", "512th", "1024th", "2048th"
     ] # Note: 2048 is supported by ly but not by MusicXML!
-    return xml_types[ly.duration.durations.index(durval)]
+    try:
+        type_index = ly.duration.durations.index(str(durval))
+    except ValueError:
+        type_index = 5
+    return xml_types[type_index]
 
 def dur2lines(dur):
     if dur == 8:
